@@ -18,13 +18,7 @@
  You should have received a copy of the GNU General Public
  License along with GimpLensfun. If not, see
  http://www.gnu.org/licenses/.
-
-###################################################################
-CHANGES:
-
-22.10.2011 - OpenMP parallelization
-20.10.2011 - Windows compatibility
-
+ 
 */
 
 #include <stdio.h>
@@ -33,14 +27,14 @@ CHANGES:
 #include <vector>
 #include <float.h>
 
-#include <lensfun.h>
+#include <lensfun/lensfun.h>
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
 #include <exiv2/image.hpp>
 #include <exiv2/exif.hpp>
 
-#define VERSIONSTR "0.2.3"
+#define VERSIONSTR "0.2.4"
 
 
 #ifndef DEBUG
@@ -274,6 +268,7 @@ unsigned long long int timespec2llu(struct timespec *ts) {
 
 //####################################################################
 // Helper functions for printing debug output
+#if DEBUG
 static void PrintMount (const lfMount *mount)
 {
     g_print ("Mount: %s\n", lf_mlstr_get (mount->Name));
@@ -303,7 +298,6 @@ static void PrintLens (const lfLens *lens)
     g_print ("\tFocal: %g-%g\n", lens->MinFocal, lens->MaxFocal);
     g_print ("\tAperture: %g-%g\n", lens->MinAperture, lens->MaxAperture);
     g_print ("\tCenter: %g,%g\n", lens->CenterX, lens->CenterY);
-    g_print ("\tCCI: %g/%g/%g\n", lens->RedCCI, lens->GreenCCI, lens->BlueCCI);
     if (lens->Mounts)
         for (int j = 0; lens->Mounts [j]; j++)
             g_print ("\tMount: %s\n", lf_db_mount_name (ldb, lens->Mounts [j]));
@@ -332,6 +326,7 @@ static void PrintLenses (const lfLens **lenses)
     else
         g_print ("\t- failed\n");
 }
+#endif
 //--------------------------------------------------------------------
 
 
@@ -358,20 +353,24 @@ static void dialog_set_cboxes( string sNewMake, string sNewCamera, string sNewLe
             return;
 
     // try to match maker with predefined list
+    int iNumMakers = 0;
     for (int i = 0; CameraMakers[i].compare("NULL")!=0; i++)
     {
         if (StrCompare(CameraMakers[i], sNewMake)==0) {
             gtk_combo_box_set_active(GTK_COMBO_BOX(maker_combo), i);
             iCurrMakerID = i;
         }
+        iNumMakers++;
     }
 
-    // return if no maker has been identified
-    if (iCurrMakerID==-1)
-        return;
-
-    // store cam maker
-    sLensfunParameters.CamMaker = CameraMakers[iCurrMakerID];
+    if (iCurrMakerID>=0)
+        sLensfunParameters.CamMaker = CameraMakers[iCurrMakerID];
+    else {
+        gtk_combo_box_append_text( GTK_COMBO_BOX(maker_combo), sNewMake.c_str());
+        gtk_combo_box_set_active(GTK_COMBO_BOX(maker_combo), iNumMakers);
+        iNumMakers++;
+        sLensfunParameters.CamMaker = sNewMake;
+    }
 
     // clear camera/lens combobox
     store = gtk_combo_box_get_model( GTK_COMBO_BOX(camera_combo) );
@@ -380,7 +379,7 @@ static void dialog_set_cboxes( string sNewMake, string sNewCamera, string sNewLe
     gtk_list_store_clear( GTK_LIST_STORE( store ) );
 
     // get all cameras from maker out of database
-    cameras = ldb->FindCamerasExt (CameraMakers[iCurrMakerID].c_str(), NULL, LF_SEARCH_LOOSE );
+    cameras = ldb->FindCamerasExt (sLensfunParameters.CamMaker.c_str(), NULL, LF_SEARCH_LOOSE );
     if (cameras) {
         for (int i=0; cameras [i]; i++){
             vCameraList.push_back(string(lf_mlstr_get(cameras[i]->Model)));
@@ -755,7 +754,7 @@ inline float Lanczos(float x)
     if ( (x<FLT_MIN) && (x>-FLT_MIN) )
         return 1.0f;
 
-    if ( (x >= cLanczosWidth) && (x <= (-1)*cLanczosWidth) )
+    if ( (x >= cLanczosWidth) || (x <= (-1)*cLanczosWidth) )
         return 0.0f;
 
     float xpi = x * static_cast<float>(M_PI);
@@ -884,6 +883,12 @@ static void process_image (GimpDrawable *drawable) {
     guchar *ImgBuffer;
     guchar *ImgBufferOut;
 
+    if ((sLensfunParameters.CamMaker.length()==0) ||
+        (sLensfunParameters.Camera.length()==0) ||
+        (sLensfunParameters.Lens.length()==0)) {
+            return;
+    }
+
     #ifdef POSIX
     struct timespec profiling_start, profiling_stop;
     #endif
@@ -1222,30 +1227,45 @@ run (const gchar*   name,
 
     imageID = param[1].data.d_drawable;
 
+    if (DEBUG) g_print ("Loading database...");
+    //Load lensfun database
+    ldb = new lfDatabase ();
+    if (ldb->Load () != LF_NO_ERROR) {
+        if (DEBUG) g_print ("failed!\n");
+    } else {
+        if (DEBUG) g_print ("OK\n");
+    }
+
+    // read exif data
+    const gchar *filename = gimp_image_get_filename(imageID);
+    if (DEBUG) g_print ("Image file path: %s\n", filename);
+
+    if ((filename == NULL) || (read_opts_from_exif(filename) != 0)) {
+	    loadSettings();
+    }
+
     run_mode = GimpRunMode(param[0].data.d_int32);
     if (run_mode == GIMP_RUN_INTERACTIVE)
     {
-        if (DEBUG) g_print ("Loading database...");
-        //Load lensfun database
-        ldb = lf_db_new ();
-        ldb->Load ();
-
-        // read exif data
-        const gchar *filename = gimp_image_get_filename(imageID);
-        if (DEBUG) g_print ("Image file path: %s\n", filename);
-
-        if ((filename == NULL) || (read_opts_from_exif(filename) != 0)) {
-            loadSettings();
-        }
-
-        if (DEBUG) g_print ("Creating dialog...\n");
-        /* Display the dialog */
-        if (create_dialog_window (drawable)) {
-            process_image(drawable);
-        }
-
-        storeSettings();
-
-        lf_db_destroy(ldb);
+	    if (DEBUG) g_print ("Creating dialog...\n");
+	    /* Display the dialog */
+	    if (create_dialog_window (drawable)) {
+		    process_image(drawable);
+	    }
+    } 
+    else 
+    {
+	    /* If run_mode is non-interactive, we use the configuration
+	     * from read_opts_from_exif. If that fails, we use the stored settings
+	     * (loadSettings()), e.g. the settings that have been made in the last 
+	     * interactive use of the plugin. One day, all settings should be 
+	     * available as arguments in non-interactive mode.
+	     */
+	    process_image(drawable);
     }
+
+    storeSettings();
+
+    ldb->Destroy();
+
 }
